@@ -29,7 +29,6 @@ use POSIX;
 use FileHandle;
 use File::Temp ();
 
-use Sbuild::Conf;
 use Sbuild::Log;
 use Sbuild::Sysconfig;
 
@@ -84,7 +83,6 @@ sub get_command_internal {
 
     my $command = $options->{'INTCOMMAND'}; # Command to run
     my $user = $options->{'USER'};          # User to run command under
-    my $chroot = $options->{'CHROOT'};      # Run in chroot?
     my $dir;                                # Directory to use (optional)
     $dir = $self->get('Defaults')->{'DIR'} if
 	(defined($self->get('Defaults')) &&
@@ -95,53 +93,48 @@ sub get_command_internal {
     if (!defined $user || $user eq "") {
 	$user = $self->get_conf('USERNAME');
     }
-    if (!defined $chroot) {
-	$chroot = 1;
-    }
 
     my @cmdline;
-    my $chdir = undef;
-    if ($chroot != 0) { # Run command inside chroot
-	if (!defined($dir)) {
-	    $dir = '/';
-	}
 
-	my $shellcommand;
-	foreach (@$command) {
-	    my $tmp = $_;
-	    $tmp =~ s/'//g; # Strip any single quotes for security
-	    if ($_ ne $tmp) {
-		$self->log_warning("Stripped single quote from command for security: $_\n");
-	    }
-	    if ($shellcommand) {
-		$shellcommand .= " '$tmp'";
-	    } else {
-		$shellcommand = "'$tmp'";
-	    }
-	}
-
-	@cmdline = ('/usr/sbin/chroot', $self->get('Location'),
-		    $self->get_conf('SU'), '-p', "$user", '-s',
-		    $Sbuild::Sysconfig::programs{'SHELL'}, '-c',
-		    "cd '$dir' && $shellcommand");
-    } else { # Run command outside chroot
-	if ($options->{'CHDIR_CHROOT'}) {
-	    my $tmpdir = $self->get('Location');
-	    $tmpdir = $tmpdir . $dir if defined($dir);
-	    $dir = $tmpdir;
-	}
-	if ($user ne $self->get_conf('USERNAME')) {
-	    $self->log_warning("Command \"$command\" cannot be run as user $user on the host system\n");
-	}
-	$chdir = $dir if defined($dir);
-	push(@cmdline, @$command);
+    if (!defined($dir)) {
+	$dir = '/';
     }
 
-    $options->{'CHROOT'} = $chroot;
+    my $shellcommand;
+    foreach (@$command) {
+	my $tmp = $_;
+	$tmp =~ s/'//g; # Strip any single quotes for security
+	if ($_ ne $tmp) {
+	    $self->log_warning("Stripped single quote from command for security: $_\n");
+	}
+	if ($shellcommand) {
+	    $shellcommand .= " '$tmp'";
+	} else {
+	    $shellcommand = "'$tmp'";
+	}
+    }
+
+    my $need_chroot = 0;
+    $need_chroot = 1
+	if ($self->get('Location') ne '/');
+
+    my $need_su = 0;
+    $need_su = 1
+	if (($need_chroot && $user ne 'root') ||
+	    (!$need_chroot && $user ne $self->get_conf('USERNAME')));
+
+    push(@cmdline, $self->get_conf('SUDO'))
+	 if ($need_chroot || $need_su);
+    push(@cmdline, '/usr/sbin/chroot', $self->get('Location'))
+	if ($need_chroot);
+    push(@cmdline, $self->get_conf('SU'), '-p', "$user", '-s')
+	if ($need_su);
+    push(@cmdline, '/bin/sh', '-c', "cd '$dir' && $shellcommand");
+
     $options->{'USER'} = $user;
     $options->{'COMMAND'} = $command;
     $options->{'EXPCOMMAND'} = \@cmdline;
-    $options->{'CHDIR'} = $chdir;
+    $options->{'CHDIR'} = undef;
     $options->{'DIR'} = $dir;
 }
 
